<?php

/**
 * @file
 * Drupad configuration
 */

function drupad_settings_form($form_state) {
  $form = array();

  // Services
  $form['drupad_services'] = array(
    '#type' => 'fieldset',
    '#title' => t('Enabled services'),
    '#description' => t("Select features you want your site to expose to the application. Users must have the permission <strong>use Drupad application</strong> to be able to login form their device. If a feature below is not enabled, it won't be shown on the application regardless of the permission, even for user 1."),
    '#collapsible' => TRUE,
  );

  foreach (drupad_available_callbacks() as $callback => $service) {

    $disabled = $service['disabled'] == TRUE ? TRUE : FALSE;

    if (!$disabled) {
      $default_value = variable_get('drupad_callback_'. $callback, 1);
    }
    else {
      variable_set('drupad_callback_'. $callback, 0);
      $default_value = 0;
    }

    $form['drupad_services']['drupad_callback_'. $callback] = array(
      '#type' => 'checkbox',
      '#title' => check_plain($service['title']),
      '#description' => filter_xss($service['description']),
      '#default_value' => $default_value,
      '#disabled' => $disabled,
    );
  }

  // User permissions
  $form['options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Other settings'),
    '#collapsible' => TRUE,
  );

  $content_types = node_get_types($op = 'types', $node = NULL, $reset = FALSE);

  $types = array();
  foreach ($content_types as $system_name => $content_type) {
    $types[$system_name] = $content_type->name;
  }

  asort($types, $sort_flags = SORT_STRING);

  $form['options']['drupad_content_types_excluded'] = array(
    '#type' => 'select',
    '#title' => t('Content types to exclude from content listing'),
    '#options' => $types,
    '#default_value' => variable_get('drupad_content_types_excluded', array()),
    '#description' => t('Select content types that you do not want Drupad to expose to the application, good for micro blogging streams. This setting does not affect node creation/editing'),
    '#multiple' => TRUE,
  );

  $form['options']['drupad_content_types_excluded_node_add'] = array(
    '#type' => 'select',
    '#title' => t('Content types to exclude from <em>Create content</em>'),
    '#options' => $types,
    '#default_value' => variable_get('drupad_content_types_excluded_node_add', array()),
    '#description' => t('Content types that shouldn\'t appear in <em>Create content</em>. Useful for <em>Panel</em>, <em>Webform</em>, etc.'),
    '#multiple' => TRUE,
  );

  $form['testing'] = array(
    '#type' => 'fieldset',
    '#title' => t('Testing / debug'),
  );

  $form['testing']['test_login'] = array(
    '#type' => 'textfield',
    '#title' => t('Username'),
  );
  $form['testing']['test_password'] = array(
    '#type' => 'password',
    '#title' => t('Password'),
  );

  $form['testing']['test'] = array(
    '#type' => 'button',
    '#value' => t('Launch the test'),
    '#ahah' => array(
      'path' => 'drupad/simulate-client',
      'wrapper' => 'drupal-debug-result',
      'method' => 'replace',
      'effect' => 'fade',
    ),
  );

  $form['testing']['wrapper'] = array(
    '#prefix' => '<div id="drupal-debug-result">',
    '#value' => '&nbsp;',
    '#suffix' => '</div>',
  );

  $form = system_settings_form($form);

  $form['#submit'][] = 'drupad_settings_form_submit';

  return $form;
}

function drupad_settings_form_submit($form, &$form_state) {
  menu_rebuild();
}